import Flutter
import UIKit
import DCDicom

public class EdDicomViewerPlugin: NSObject, FlutterPlugin {
    
    let fileManager = FileManager.default
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "ed_dicom_viewer", binaryMessenger: registrar.messenger())
        let instance = EdDicomViewerPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "getViewDicom":
            let args = call.arguments as! Dictionary<String, Any>
            do {
                let fileData = try Data(contentsOf: URL(fileURLWithPath: args["filePath"]! as! String))
                if !isDICOM(data: fileData) {
                    return
                }
                let dicomObject = try DicomObject(data: fileData)
                
                
                let pixelData = dicomObject[0x7FE00010]
                let widthData = dicomObject[0x00280011]!.rawValue
                let heightData = dicomObject[0x00280010]!.rawValue
                let width = widthData!.withUnsafeBytes { $0.load(as: UInt16.self)}
                let height = heightData!.withUnsafeBytes { $0.load(as: UInt16.self)}
                let patientName = dicomObject[0x00100010] == nil ? "" : String(data:dicomObject[0x00100010]!.rawValue!,encoding:.isoLatin1)
                let patientGender = dicomObject[0x00100040] == nil ? "" : String(data:dicomObject[0x00100040]!.rawValue!,encoding:.isoLatin1)
                let patientID = dicomObject[0x00100020] == nil ? "" : String(data:dicomObject[0x00100020]!.rawValue!,encoding:.isoLatin1)
                let patientAge = dicomObject[0x00101010] == nil ? "" : String(data:dicomObject[0x00101010]!.rawValue!,encoding:.isoLatin1)
                let patientBirthDate = dicomObject[0x00100030] == nil ? "" : String(data:dicomObject[0x00100030]!.rawValue!,encoding:.isoLatin1)
                let patientWeight = dicomObject[0x00101030] == nil ? "" : String(data:dicomObject[0x00101030]!.rawValue!,encoding:.isoLatin1)
                let patientHeight = dicomObject[0x00101020] == nil ? "" : String(data:dicomObject[0x00101020]!.rawValue!,encoding:.isoLatin1)
                print(">>> \(width),\(height)")
                let image = createImage(from: pixelData!.rawValue!, width: Int(width), height: Int(height))
                print(">>>>>>> im \(image?.jpegData(compressionQuality: 100))")
                let responseModel = ResponseModel(decodedBytes:image!.jpegData(compressionQuality: 100)!.base64EncodedString(), patientName: patientName!, patientGender: patientGender!, patientID: patientID!, patientAge: patientAge!, patientBirthDate: patientBirthDate!, patientWeight: patientWeight!, patientHeight: patientHeight!)
                result(JsonUtil.convertToJson(responseModel: responseModel))
            } catch  {
                
            }
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    
    func isDICOM(data: Data) -> Bool {
        let minimumLength = 132
        let dicomSignatureOffset = 128
        let dicomSignature = "DICM".data(using: .ascii)!
        
        guard data.count >= minimumLength else {
            return false
        }
        
        let signatureData = data.subdata(in: dicomSignatureOffset..<dicomSignatureOffset+4)
        
        return signatureData == dicomSignature
    }
    
    
    func createImage(from pixelData: Data, width: Int, height: Int) -> UIImage? {
        let bitsPerComponent = 8
        let bitsPerPixel = 8
        let bytesPerRow = width
        
        guard let providerRef = CGDataProvider(data: pixelData as CFData) else {
            return nil
        }
        
        guard let cgim = CGImage(
            width: width,
            height: height,
            bitsPerComponent: bitsPerComponent,
            bitsPerPixel: bitsPerPixel,
            bytesPerRow: bytesPerRow,
            space: CGColorSpaceCreateDeviceGray(),
            bitmapInfo: CGBitmapInfo(rawValue: CGImageAlphaInfo.none.rawValue),
            provider: providerRef,
            decode: nil,
            shouldInterpolate: true,
            intent: CGColorRenderingIntent.defaultIntent
        ) else {
            return nil
        }

        return UIImage(cgImage: cgim)
    }
}

extension Data {
    
    subscript<T: BinaryInteger>(at offset: Int, convertEndian convertEndian: Bool = false) -> T? {
        value(ofType: T.self, at: offset, convertEndian: convertEndian)
    }
    
    func value<T: BinaryInteger>(ofType: T.Type, at offset: Int, convertEndian: Bool = false) -> T? {
        let right = offset &+ MemoryLayout<T>.size
        guard offset >= 0 && right > offset && right <= count else {
            return nil
        }
        let bytes = self[offset ..< right]
        if convertEndian {
            return bytes.reversed().reduce(0) { T($0) << 8 + T($1) }
        } else {
            return bytes.reduce(0) { T($0) << 8 + T($1) }
        }
    }
}
